---------------------------------------------------------------------------------------------------
--
-- Title       : serial
-- Design      : serial
-- Author      : Ney Calazans
-- Company     : FACIN-PUCRS
--
---------------------------------------------------------------------------------------------------
--
-- File        : serial.vhd
-- Generated   : Tue Nov  4 15:18:17 2003
-- From        : interface description file
-- By          : Itf2Vhdl ver. 1.20
--
---------------------------------------------------------------------------------------------------
--
-- Description : This hardware implements a simple serial interface with an
-- RS232-like interface
--
---------------------------------------------------------------------------------------------------

--{{ Section below this comment is automatically maintained
--   and may be overwritten
--{entity {serial} architecture {serial}}

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_unsigned.all;

entity serial is
	 port(
		 reset : in STD_LOGIC;
		 clock : in STD_LOGIC;
		 send : in STD_LOGIC;
		 word : in STD_LOGIC_VECTOR(7 downto 0);
		 busy : out STD_LOGIC;
		 line : out STD_LOGIC
	     );
end serial;

--}} End of automatically maintained section

architecture serial of serial is 
type states is (s0, S1, S2, S3);
signal EA, PE: states;
signal hab_read_word, hab_count, reset_count: std_logic;
signal pint: std_logic_vector(7 downto 0); 
signal count:integer range 0 to 8;
begin
	
count_p: process (clock, reset)	 -- a module-8 counter 
begin	
	if reset='1' or reset_count='1' then count<=0;
	elsif clock'event and clock='1' then
		if hab_count='1' then count<=count+1;
		end if;
	end if;
end process;

pint_p: process (clock, reset)	 -- a register to keep the byte payload
begin	
	if reset='1' then pint<=(others=>'0');
	elsif clock'event and clock='1' then
		if hab_read_word='1' then
			pint<=word;
		end if;
	end if;
end process;


seq_part: process(reset,clock)	
begin 
	if reset='1' then
		EA <= S0;
	elsif clock'event and clock='1' then
		EA <= PE;
	end if;
end process;

comb_part: process (reset, word, send, EA, count)	
begin
   hab_read_word<='0'; hab_count<='0'; 
   reset_count<='0';
   case EA is
      when S0 =>
	  if reset='1' then PE <= S0;
	  else PE<=S1;	
	  end if;
	  busy<='1'; line<='1';	
	  
      when S1 =>
	  if send='0' then PE<=S1;
	  else PE<=S2;
	  end if;
	  busy<='0';
	  line<='1';
	  
	  when S2 =>
	  PE<=S3;
	  hab_read_word<='1' ;
	  reset_count<='1';
	  busy<='1';
	  line<='0';
	  
	  when S3 =>
	  if count=7 then PE<=S1;
	  else PE<=S3; 
		  hab_count<='1';
	  end if;
	  busy<='1'	  ;
	  line<=pint(count);
	  
      when others =>
         null;
   end case;
   -- example
end process;


end serial;
